(function ($, Drupal, drupalSettings) {
  'use strict';
  Drupal.behaviors.product_gallery = {
    attach: function (context, settings) {
      once('product_gallery', '.gallery-container', context).forEach((thisImg) => {
        const $container = $(thisImg);

        const $mainContainer = $container.find('.main-image-container');
        let $currentImg = $container.find('.current-image');
        const $magnifier = $container.find('.image-magnifier');
        let currentIndex = 0;
        let defaultZoomScale = $container.attr('data-default-zoom-scale');
        let allowThumbnailOverlap = ['true', '1'].includes($container.attr('data-allow-thumbnail-overlap'));
        let thumbnailOverlapThreshold = parseInt($container.attr('data-thumbnail-overlap-threshold'), 10);
        thumbnailOverlapThreshold = isNaN(thumbnailOverlapThreshold) ? 5 : thumbnailOverlapThreshold;

        let enableMouseZoom = ['true', '1'].includes($container.attr('data-enable-mouse-zoom'));
        let currentScale = (defaultZoomScale !== undefined && defaultZoomScale !== '') ? parseFloat(defaultZoomScale) : 2;

        function getResponsiveSrc($el) {
          const width = window.innerWidth;
          return width <= 576 ? $el.data('small-src')
            : width <= 992 ? $el.data('medium-src')
              : $el.data('large-src');
        }

        function getThumbSrc($el) {
          return $el.data('thumb-src');
        }

        function setMainImage($sourceEl) {
          const newSrc = getResponsiveSrc($sourceEl);
          $currentImg.css('background-image', `url(${newSrc})`);
          $currentImg.data({
            'large-src': $sourceEl.data('large-src'),
            'medium-src': $sourceEl.data('medium-src'),
            'small-src': $sourceEl.data('small-src')
          });
        }

        function createSlideImage($sourceEl, direction) {
          const newSrc = getResponsiveSrc($sourceEl);
          const offset = direction === 'right' ? '100%' : '-100%';

          const $newImg = $('<div class="image-slide">').css({
            backgroundImage: `url(${newSrc})`,
            transform: `translateX(${offset})`
          }).data({
            'large-src': $sourceEl.data('large-src'),
            'medium-src': $sourceEl.data('medium-src'),
            'small-src': $sourceEl.data('small-src')
          });

          $mainContainer.append($newImg);
          void $newImg[0].offsetWidth;

          $currentImg.css('transform', `translateX(${direction === 'right' ? '-100%' : '100%'})`);
          $newImg.css('transform', 'translateX(0)');

          setTimeout(() => {
            $currentImg.remove();
            $newImg.attr('id', 'current-image');
            $currentImg = $newImg;
          }, 400);
        }

        const $thumbs = $container.find('.image-thumb');
        if (allowThumbnailOverlap) {
          const $thumbs = $container.find('.image-thumb');
          if ($thumbs.length > thumbnailOverlapThreshold) {
            $container.find('.thumbnail-container').addClass('overlap');
          }
        }

        $thumbs.each(function () {
          const bg = getThumbSrc($(this));
          $(this).css('background-image', `url(${bg})`);
        });

        setMainImage($container.find('.image-thumb.active'));

        $thumbs.on('click', function () {
          currentScale = (defaultZoomScale !== undefined && defaultZoomScale !== '') ? parseFloat(defaultZoomScale) : 2;
          const newIndex = parseInt($(this).data('index'));
          if (newIndex === currentIndex) { return;
          }

          const direction = newIndex > currentIndex ? 'right' : 'left';
          createSlideImage($(this), direction);

          $thumbs.removeClass('active');
          $(this).addClass('active');

          $container.find('.image-thumb').css('border', '');
          $container.find('.image-thumb').css('border-color', '');

          const borderColor = $(this).data('border-color');
          // Check if borderColor is set and not empty
          if (borderColor) {
            $(this).css('border', '2px solid');
            $(this).css('border-color', borderColor);
          }

          currentIndex = newIndex;
        });

        // $container.find('.main-image').on('mousemove', function (e) {
        //   if (window.innerWidth < 768) return;

        //   if ($magnifier.length) {
        //     const offsetX = e.offsetX;
        //     const offsetY = e.offsetY;
        //     const width = $mainContainer.width();
        //     const height = $mainContainer.height();

        //     let magnifierX = offsetX - $magnifier.width() / 2;
        //     let magnifierY = offsetY - $magnifier.height() / 2;

        //     magnifierX = Math.max(0, Math.min(magnifierX, width - $magnifier.width()));
        //     magnifierY = Math.max(0, Math.min(magnifierY, height - $magnifier.height()));

        //     $magnifier.css({
        //       top: magnifierY,
        //       left: magnifierX,
        //       display: 'block'
        //     });
        //   }

        //   $currentImg.css({
        //     transformOrigin: `${e.offsetX}px ${e.offsetY}px`,
        //     transform: `scale(${currentScale})`
        //   });
        // });

        // $container.find('.main-image').on('mouseleave', function () {
        //   if ($magnifier.length) {
        //     $magnifier.hide();
        //   }
        //   $currentImg.css('transform', 'scale(1)');
        // });

        $container.find('.main-image').on('mousemove touchmove', function (e) {
          // Prevent default touch behavior
          e.preventDefault();

          if (window.innerWidth < 768 && e.type === 'touchmove') { return;  // Only handle touchmove on small screens
          }

          // Get the position of the mouse or touch point
          let offsetX, offsetY;

          // For touch events, we use the first touch point
          if (e.type === 'touchmove') {
            offsetX = e.originalEvent.touches[0].pageX - $container.offset().left;
            offsetY = e.originalEvent.touches[0].pageY - $container.offset().top;
          } else {
            offsetX = e.offsetX;
            offsetY = e.offsetY;
          }

          if ($magnifier.length) {
            const width = $mainContainer.width();
            const height = $mainContainer.height();

            let magnifierX = offsetX - $magnifier.width() / 2;
            let magnifierY = offsetY - $magnifier.height() / 2;

            magnifierX = Math.max(0, Math.min(magnifierX, width - $magnifier.width()));
            magnifierY = Math.max(0, Math.min(magnifierY, height - $magnifier.height()));

            $magnifier.css({
              top: magnifierY,
              left: magnifierX,
              display: 'block'
            });
          }

          $currentImg.css({
            transformOrigin: `${offsetX}px ${offsetY}px`,
            transform: `scale(${currentScale})`
          });
        });

        $container.find('.main-image').on('mouseleave touchend touchcancel', function () {
          if ($magnifier.length) {
            $magnifier.hide();
          }
          $currentImg.css('transform', 'scale(1)');
        });

        $(window).on('resize', function () {
          $thumbs.each(function () {
            $(this).css('background-image', `url(${getResponsiveSrc($(this))})`);
          });
          setMainImage($container.find('.image-thumb.active'));
        });

        $container.find('.image-thumb').hover(
          function () {
            const borderColor = $(this).data('border-color');
            // Check if borderColor is set and not empty
            if (borderColor) {
              $(this).css('border', '2px solid');
              $(this).css('border-color', borderColor);
            }
          },
          function () {
            if (!$(this).hasClass('active')) {
              $(this).css('border', '');
              $(this).css('border-color', ''); // Reset the border color when hover ends
            }
          }
        );

        if (enableMouseZoom) {
          $container.find('.main-image').on('wheel', function (e) {
            e.preventDefault();
            if (e.originalEvent.deltaY > 0) {
              currentScale = Math.max(1, currentScale - 0.1);
            } else {
              currentScale = Math.min(3, currentScale + 0.1);
            }

            $currentImg.css({
              transform: `scale(${currentScale})`,
              transition: 'transform 0.2s ease'
            });
          });
        }
      });
    }
  };
})(jQuery, Drupal, drupalSettings);
